/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */


#define FLXVI_VERSION_REV 8
#define FLXVI_VERSION_DATE "2017-05-18"

#include "flxvi.h"


#if defined(_MSC_VER)
  #include "flxvi_export.h"
#else
  #define FLXVI_EXPORT
#endif

#ifdef __WINDOWS__
  #pragma comment(lib, "flxlib")
#endif

FlxViManager* viManager = NULL;

// define some output-streams (for buffering)
    std::ostringstream flxOutputStream;
    std::ostringstream& flxErrorStream = flxOutputStream;

extern "C" FLXVI_EXPORT void flxvi_init( FesslixMain* flxptr ) {
  GlobalVar.slog(4) << "  FlxVi-interface - Version " << FLXVI_VERSION_REV << " (" << FLXVI_VERSION_DATE << ")" << std::endl;
  // set pointer
    FlxViObj_Flx::set_fesslix_ptr(flxptr);
  // re-initialize streams
    GlobalVar.set_stdcerr(&flxErrorStream);
    GlobalVar.set_slogcout(GlobalVar.get_log(),&flxOutputStream);
  // Start the GUI
    viManager = new FlxViManager(flxOutputStream, flxErrorStream);
  // print GNU
    print_GNU(flxOutputStream);
  // Prepare the log
    viManager->do_log();
  // start the command-input mode
    viManager->execute();
}

extern "C" FLXVI_EXPORT void flxvi_close() {
  if (viManager) delete viManager;
}



// ---------------------------------------------------------------------------------------------------------


FlxViManager::FlxViManager(std::ostringstream& flxOutputStream, std::ostringstream& flxErrorStream)
:flxOutputStream(flxOutputStream), flxErrorStream(flxErrorStream)
{
  FlxViDialogs::set_viManager(this);
  
  initscr();				// Start curses mode
  getmaxyx(stdscr,NC_ROW,NC_COL);	// get the number of rows and columns

  raw();				// Line buffering disabled
  keypad(stdscr, TRUE);			// We get F1, F2 etc..
  noecho();				// Don't echo() while we do getch
  
  start_color();			// Start the color mode
    init_pair(1,COLOR_WHITE,COLOR_BLACK);	// defines the default color
    init_pair(2,COLOR_WHITE,COLOR_BLUE);	// default color for menus
    init_pair(3,COLOR_BLACK,COLOR_YELLOW);	// default color for active regions
    init_pair(4,COLOR_WHITE,COLOR_RED);		// in case of errors
    init_pair(5,COLOR_BLACK,COLOR_WHITE);	// dialog-settings
    
  curs_set(0);
  refresh();
  
  // Create the Fesslix-Version-Window
    viTitle = new FlxViTitle(0,0,1,NC_COL,flx_get_version(),FlxViManager::get_version());
  // Create the vi-CMD-Window
    viCmdBar = new FlxViCmdBar(NC_ROW-1,0,NC_COL);
  // Create the logging-Window
    viLog = new FlxViLog(1,0,NC_ROW-4,NC_COL);
  // Create the Fesslix-Input
    viFlxInput = new FlxVi_FlxInput(NC_ROW-4,0,3,NC_COL,flxErrorStream);  
    
  // Update the hole screen
    updateScreen();
    
  // Log what has been computed so far (in Fesslix)
    do_log();
}

FlxViManager::~FlxViManager()
{
  // delete windows
    delete viTitle;
    delete viLog;
    delete viFlxInput;
    delete viCmdBar;
    
  endwin();			// End curses mode
}

void FlxViManager::execute()
{
  std::string strcmd="";
  FlxViObj_Base* cWin;
  while (true) {
    try {
      cWin=NULL;
      // read the (top-level) command
	if (strcmd.empty()) {
	  viCmdBar->input_loop(strcmd,true);
	}
      // parse 'strcmd'
	// reset errors
	  viCmdBar->reset_error();
	// the available options
	  if (strcmd=="q") {			// ask for quit
	    if (viDlg.DlgQuit()) return;
	  } else if (strcmd=="q!") {		// quit without asking first
	    return;
	  } else if (strcmd=="l") {		// goto log-mode
	    cWin = viLog;
	  } else if (strcmd=="i") {		// goto input-mode
	    cWin = viFlxInput;
	  } else {				// ERROR: unknown command
	    viCmdBar->vi_cmd_error("FlxViManager::execute","'" + strcmd + "' is not a valid command");
	  }
	if (cWin==NULL) {
	  strcmd="";
	} else {
	  strcmd = cWin->activate();		// activate the requested 'window'
	}
    }
    catch ( FlxException_NeglectInInteractive &e) {
      flxErrorStream << std::endl << e.what() << std::endl;
      GlobalVar.logLevel_strong_reset();
      if (GlobalVar.check_logNOTcout()) {
	GlobalVar.slog(1) << std::endl << e.what() << std::endl;
      }
    }
  }
}

void FlxViManager::updateScreen()
{
  viTitle->updateWindow();
  viLog->updateWindow();
  viFlxInput->updateWindow();
  viCmdBar->updateWindow();
  doupdate();
}

void FlxViManager::throw_CMDerror(const std::string& errIndex, const std::string& errmsg)
{
  viCmdBar->vi_cmd_error(errIndex, errmsg);
}

void FlxViManager::do_log()
{
  viLog->do_log(flxOutputStream.str());
  flxOutputStream.str("");
}

std::string FlxViManager::get_version()
{
  std::ostringstream ssV;
  ssV << "trunk." << FLXVI_VERSION_REV << " (" << FLXVI_VERSION_DATE << ")";
  return ssV.str();
}

void FlxViManager::reset_error()
{
  viCmdBar->reset_error();
}

